/**
  bad_cast example

  Type of the exceptions thrown by dynamic_cast
  when it fails the run-time check performed
  on references to polymorphic class types.

  The run-time check fails if the object
  would be an incomplete object of the destination type.

  Its member what returns a null-terminated
  character sequence identifying the exception.
  */

#include <iostream>       // std::cout
#include <typeinfo>       // std::bad_cast

class Base
{
    virtual void member() {}
};

class Derived: Base {};

int main()
{
    try
    {
        Base b;
        Derived &rd = dynamic_cast<Derived &>(b);
    }

    // type of exception is wrong here:
    catch(const std::bad_typeid &bt)
    {
        std::cerr << "bad_typeid caught: " << bt.what() << std::endl;
    }

    // caught here instead (parent exception class)
    catch(const std::exception &bt)
    {
        std::cerr << "parent caught: " << bt.what() << std::endl;
    }

    // Will catch non- std::exception exceptions
    catch(...)
    {
        std::cerr << "Something else: " << std::endl;
    }

    return 0;
}

